# asmdraw.asm
# (!) syntax is for GNU ARM assembler
#

    .section text, "x"
    
    .global asm_draw_block
	.global asm_draw_tblock
    .global asm_draw_trans
    .global asm_upscale_ffb
    .global asm_noscale_ffb


# draws color block from sprite pointed by spraddr to bufaddr (zero color pixels will not be painted)
# color points are WORDs with 454 color mask
#
asm_draw_block:

    mov r12, sp					@ standard setup code
    stmfd sp!, {r4-r11,lr}		@ 

	@ r0 - bufaddr
	@ r1 - spraddr
	@ r2 - DX (high word), DY (low word) - sizes of block to draw (NOT size of sprite itself!)
	@ r3 - sprite's DX

	mov     r4, #8192			@ r4 <- 0x1FFF
	sub     r4, r4, #1			@ it's a mask for clearing upper 3 bits

	mov r3, r3, LSL #1			@ multiply sprite's DX by 2 because color data are WORDs

	mov r7, r2, LSL #16			@ r7 <- low word of r2 (<dy> to draw)
	mov r7, r7, LSR #16			@ 
asm_draw_block_ycycle:
    mov r10, r0					@ restore bufaddr position
    mov r11, r1					@ restore spraddr position

    mov   r6, r2, LSR #16		@ r6  <- high word of r2 (<dx> to draw)
asm_draw_block_xcycle:
    ldrh    r12, [r11], #2		@ color word from [spraddr] and increment spraddr by #2
	ands    r12, r12, r4        @ clears upper 3 bits and comparing to zero
    strneh  r12, [r10]			@ store that color word to [bufaddr] if it isn't zero (conditional execution, nice feature of arm instructions)
	add		r10, r10, #2		@ increment bufaddr by WORD
	subs    r6, r6, #1			@ loop while dy is positive
	bne asm_draw_block_xcycle	

    add   r0, r0, #480			@ advance bufaddr to next row (buffer width: 240)
	add   r1, r1, r3			@ advance spraddr to next row

    subs  r7, r7, #1			@ loop while dy is positive
    bne asm_draw_block_ycycle	@
    
    ldmfd sp!, {r4-r11,pc}		@ standard cleanup code + return


    
# draws color block from sprite pointed by spraddr to bufaddr
# 12-bit colors will be transformed with 5-th argument's table
# (mainly for highlight effects)
#
asm_draw_tblock:

    mov r12, sp
    stmfd sp!, {r4-r11,lr}

	@ r0 - bufaddr
	@ r1 - spraddr
	@ r2 - DX (high word), DY (low word) - sizes of block to draw (NOT size of sprite itself!)
	@ r3 - sprite's DX

    ldr		r5, [r12]			@ 5'th argument - colors transformation table

	mov r3, r3, LSL #1			@ multiply sprite's DX by 2 because color data are WORDs

	mov r7, r2, LSL #16			@ r7 <- low word of r2 (<dy> to draw)
	mov r7, r7, LSR #16			@ 

	mov     r4, #16384			@ r4 <- 0x3FFE - for masking
	sub     r4, r4, #2			@ we will need ((r12 & 0x1FFF) << 1) that's equal to ((r12<<1) & 0x3FFE)

asm_draw_tblock_ycycle:
    mov r10, r0					@ restore bufaddr position
    mov r11, r1					@ restore spraddr position
    
    mov   r6, r2, LSR #16		@ r6  <- high word of r2 (<dx> to draw)
asm_draw_tblock_xcycle:
    ldrh    r12, [r11], #2		@ color word from [spraddr] and increment spraddr by #2
	ands    r12,r4,r12,LSL #1   @ multiply by 2 and comparing to zero
	ldrneh  r12, [r5, r12]      @ if not zero - load transformed color r12 <- [r4 + r12]
    strneh  r12, [r10]			@ if not zero - store that color word to [bufaddr]
	add		r10, r10, #2		@ increment bufaddr by WORD
	subs    r6, r6, #1			@ loop while dy is positive
	bne asm_draw_tblock_xcycle	@

    add   r0, r0, #480			@ advance bufaddr to next row (buffer width: 240)
	add   r1, r1, r3			@ advance spraddr to next row

    subs  r7, r7, #1			@ loop while dy is positive
    bne asm_draw_tblock_ycycle	@

    ldmfd sp!, {r4-r11,pc}


    
# draws color block from sprite pointed by spraddr to bufaddr
# color points are WORDs with 3454 (argb) color mask
# full transparency support (upper 3 bits of source color word - transparency koeff, 0 - opaque, 7 - most transparent)
#
asm_draw_trans:

    mov r12, sp
    stmfd sp!, {r4-r11,lr}

	@ r0 - bufaddr
	@ r1 - spraddr
	@ r2 - DX (high word), DY (low word) - sizes of block to draw (NOT size of sprite itself!)
	@ r3 - sprite's DX
	
    ldr		r4, [r12]				@ 5'th argument - transparency colors transformation table

	mov		r3, r3, LSL #1			@ multiply sprite's DX by 2 because color data are WORDs

	mov		r7, r2, LSL #16			@ r7 <- low word of r2 (<dy> to draw)
	mov		r7, r7, LSR #16			@ 

	mov     r5, #8192				@ r5 <- 0x1FFF - for masking
	sub     r5, r5, #1				@

asm_draw_trans_ycycle:
    mov		r10, r0					@ restore bufaddr position
    mov		r11, r1					@ restore spraddr position
    mov		r6, r2, LSR #16			@ r6  <- high word of r2 (<dx> to draw)

asm_draw_trans_xcycle:

    ldrh    r12, [r11], #2			@ color word from [spraddr] and increment it's pointer
	mov     r8,  r12, LSR #13		@ r8 -> upper 3 bits (alpha channel value)
	and     r12, r12, r5		    @ clears that upper 3 bits

    ldrh    r9, [r10]				@ second color (destination pixel)
	orr     r9, r9, r8, LSL #13		@ adding (upper) 3 bits of alpha channel (from source color)
	mov     r9, r9, LSL #1			@ (damn, scaled addressing like "ldr r9, [r4, r9, LSL #1]" must be better, but not working on my xscale)
	ldrh    r9, [r4, r9]			@ gets transformed framebuffer color

	add     r12, r12, r9			@ summ both colors to r12
    strh	r12, [r10], #2			@ store that color word to [bufaddr] and increment it's pointer

	subs    r6, r6, #1				@ loop while dy is positive
	bne asm_draw_trans_xcycle		@

    add		r0, r0, #480			@ advance bufaddr to next row (buffer width: 240)
	add		r1, r1, r3				@ advance spraddr to next row

    subs	r7, r7, #1				@ loop while dy is positive
    bne asm_draw_trans_ycycle		@

    ldmfd sp!, {r4-r11,pc}


    

# copy memory block with upscaling 320x240 -> 640x480
# and color transformation
#
asm_upscale_ffb:

    mov r12, sp
    stmfd sp!, {r4-r11,lr}

	@ r0 - dstaddr
	@ r1 - srcaddr
	@ r2 - color transformation table address
	
	mov     r11, r2					@ color transformation table is [r11]

	mov		r2, r0					@ r2 <- r0 + 960 (one line below)
	add		r2, r2, #960			@

	mov     r4, #16384				@ r4 <- 0x3FFE - for masking (see below)
	sub     r4, r4, #2				@

	mov		r7, #320				@ width of our shadow buffer
asm_upscale_ffb_c1:
	mov		r6, #240				@ height of our shadow buffer
asm_upscale_ffb_c2:
	ldrh	r12, [r1], #2			@ word from [srcaddr] - 454 color
	and     r12, r4, r12, LSL #1    @ r12 = ((r12 & 0x1FFF) * 2) = (r12 * 2) & 0x3FFE
	                                @ clears upper 3 bits and multiply by 2 to get an offset for source color in transformation table
									@ well... we aren't needed to clear upper bits if shadow framebuffer isn't contains them, but in case of fire - do this here
	ldrh    r12, [r11, r12]			@ r12 <- 565 color from a conversion table in [r11]
	strh	r12, [r0], #2			@ store in [dstaddr]
	strh	r12, [r0], #2			@ store in [dstaddr+2]
	strh	r12, [r2], #2			@ store in [dstaddr+960]
	strh	r12, [r2], #2			@ store in [dstaddr+962]

	subs	r6, r6, #1				@ loop by screen DX (240)
	bne		asm_upscale_ffb_c2

	add		r0, r0, #960			@ move both destination pointers to next lines
	add		r2, r2, #960			@

	subs	r7, r7, #1				@ looping by screen dy (320)
	bne		asm_upscale_ffb_c1	

    ldmfd sp!, {r4-r11,pc}



# copy memory block without scaling but with color transformation
#
asm_noscale_ffb:

    mov r12, sp
    stmfd sp!, {r4-r11,lr}

	@ r0 - dstaddr
	@ r1 - srcaddr
	@ r2 - color transformation table address
	
	mov     r11, r2					@ color transformation table is [r11]

	mov		r7, #300				@ r7 <- 320 x 240 = 76800 = 0x12C00
	mov     r7, r7, LSL #8			@

	mov     r4, #16384				@ r4 <- 0x3FFE - for masking (see below)
	sub     r4, r4, #2				@

asm_noscale_ffb_c1:
	ldrh	r12, [r1], #2			@ word from [srcaddr] - 454 color
	and     r12, r4, r12, LSL #1    @ r12 = ((r12 & 0x1FFF) * 2) = (r12 * 2) & 0x3FFE
	                                @ clears upper 3 bits and multiply by 2 to get an offset for source color in transformation table
	ldrh    r12, [r11, r12]			@ r12 <- 565 color from a conversion table in [r11]
	strh	r12, [r0], #2			@ store in [dstaddr]

	subs	r7, r7, #1				@ loop by screen size
	bne		asm_noscale_ffb_c1

    ldmfd sp!, {r4-r11,pc}



    .end
