//
// hud.cpp
// Classes for display info, radio talks, and such topmost informational objects
//

#include "stdafx.h"


//////////////////////////////////////////////////////////////////////////
// HUD class
//////////////////////////////////////////////////////////////////////////

TIface_hud :: TIface_hud ( TSurface * sf ) : TIfaceElement( sf )
{
    // TODO: something
}
	

void TIface_hud :: Draw ( void )
{
    TCHAR s[20];
    RECT  crect;
    int   i;

    // Score
    wsprintf(s, L"%05u", surface->Score);
    draw_string(shadow_buffer, 5, 5, crect_fullscreen, s, surface->images[IMAGE_FONTS],1, 10,17);
    // Highscore
    wsprintf(s, L"%05u", surface->HighScore);
    draw_string(shadow_buffer, screen_dx-55, 5, crect_fullscreen, s, surface->images[IMAGE_FONTS],1, 10,17);

    // Energy bar
    draw_sprite(shadow_buffer, 70, 8, crect_fullscreen, surface->images[IMAGE_ENERGY_BAR],1, FALSE, FALSE);

    if ((surface->game_elements[GM_INDEX_MAINSHIP] != NULL) &&
		(surface->game_elements[GM_INDEX_MAINSHIP]->energy > 0))
	{
      // Energy bar is 0..100, but we will draw only first 98 points of energy
      i = surface->game_elements[GM_INDEX_MAINSHIP]->energy;
      if (i<0) i = 0; else if (i > 98) i = 98;
      crect.left = 71;
      crect.top = 9;
      crect.right = crect.left + i;
      crect.bottom = crect.top + surface->images[IMAGE_ENERGY_BAR]->dy[0] - 2;
      draw_sprite(shadow_buffer, 70, 8, crect, surface->images[IMAGE_ENERGY_BAR],0, FALSE, FALSE);
    }
}


//////////////////////////////////////////////////////////////////////////
// Radio class
////////////////////////////////////////////////////////////////////////

TGame_radio_node :: TGame_radio_node ( TGame_radio * fparent, TGame_radio_node * fprev )
{
    state = -1;
    parent = fparent;
    prev = fprev;
    next = NULL;
}

TGame_radio_node :: ~TGame_radio_node ()
{
    if (prev != NULL) prev->next = next;
    if (next != NULL) next->prev = prev;
    if (parent->first == this) parent->first = next;
}

void TGame_radio_node :: Draw ( void )
{
    int j, k;

    if (state < 0) return;

    // title shading rectangle
    draw_sprite(shadow_buffer, (int)title_x, (int)title_y, crect_fullscreen, parent->surface->images[IMAGE_RADIOTILES], 0, FALSE/*centered*/, FALSE/*hlight*/);
    // string - title
    draw_string(shadow_buffer, (int)title_x+3, (int)title_y+3, crect_fullscreen, title.data(), parent->surface->images[IMAGE_FONTS],0, 6,8);

    // text shading rectangle
    draw_sprite(shadow_buffer, (int)text_x, (int)text_y, crect_fullscreen, parent->surface->images[IMAGE_RADIOTILES], 1, FALSE/*centered*/, FALSE/*hlight*/);
    for (j=1, k=11; j<text_lines; j++, k+=9)
      draw_sprite(shadow_buffer, (int)text_x, (int)text_y+k, crect_fullscreen, parent->surface->images[IMAGE_RADIOTILES], 2, FALSE/*centered*/, FALSE/*hlight*/);
    draw_sprite(shadow_buffer, (int)text_x, (int)text_y+k, crect_fullscreen, parent->surface->images[IMAGE_RADIOTILES], 3, FALSE/*centered*/, FALSE/*hlight*/);
    // string - text
    draw_string(shadow_buffer, (int)text_x+3, (int)text_y+3, crect_fullscreen, text.data(), parent->surface->images[IMAGE_FONTS],0, 6,8);

    // face image
    if (head_hlight == 1)
      draw_sprite(shadow_buffer, (int)head_x, (int)head_y, crect_fullscreen, parent->surface->images[IMAGE_RADIOFACES], 0, FALSE/*centered*/, FALSE/*hlight*/);
    else
      draw_sprite(shadow_buffer, (int)head_x, (int)head_y, crect_fullscreen, parent->surface->images[IMAGE_RADIOFACES], head_sprnum, FALSE/*centered*/, FALSE/*hlight*/);
}

void TGame_radio_node :: TimerTick ( void )
{
    int j;

    if (state >= 0) {
      // in-table state
      if (state < 10) {
        j = state * 3;
        head_hlight = table_radio_flyin[j++];
        title_x = table_radio_flyin[j++];
        text_x  = table_radio_flyin[j++];
        state++;
      } else {
        state++;
        if (state >= 100) state = -1;
      }
    }
}


TGame_radio :: TGame_radio ( TSurface * sf ) : TGameElement( sf )
{
    first = NULL;
}

TGame_radio :: ~TGame_radio ()
{
    while (first != NULL) delete first;
}

void TGame_radio :: Talk (int fsprnum, tstring ftitle, tstring ftext)
{
    int __y;
    TGame_radio_node *p, *q;

    // one replica for one talking head
    p = first;
    while (p != NULL) {
        q = p->next;
        if (p->head_sprnum == fsprnum) delete p;
        p = q;
    }

    __y = 60;
    p = first;
    while (p != NULL) {
        __y = (int)p->head_y + p->text_lines*9 + 6 + 30;
        if (p->text_lines < 2) __y += 9;
        p = p->next;
    }

    surface->sounds->PlaySample(2);

    q = new TGame_radio_node(this, NULL);
    q->state = 0;
    q->head_x = 10;
    q->head_y = __y;
    q->head_hlight = 0;
    q->head_sprnum = fsprnum;
    q->title_x = 0;
    q->title_y = q->head_y;
    q->title = ftitle;
    q->text_x = -100;
    q->text_y = q->title_y + 18;
    q->text = ftext;
    q->text_lines = simple_word_wrap(&(q->text), 27);

    if (first == NULL) first = q;
    else {
      p = first; while (p->next != NULL) p = p->next;
      p->next = q;
      q->prev = p;
    }
}

void TGame_radio :: Draw (void)
{
    TGame_radio_node *p;
    p = first;
    while (p != NULL) {
        p->Draw();
        p = p->next;
    }
}

void TGame_radio :: TimerTick (void)
{
    TGame_radio_node *p, *q;
    p = first;
    while (p != NULL) {
        q = p->next;
        p->TimerTick(); if (p->state < 0) delete p;
        p = q;
    }
}
