//
// mainmenu.cpp
// Class for main menu processing
//

#include "stdafx.h"


// constructor
//
TMainMenu :: TMainMenu ( TSurface * sf )
{
    surface = sf;
    menu_level = 0; // menu starts with animation 
    menu_tick  = 0;
    img_bgr = NULL;
    img_all = NULL;
}

// destructor
//
TMainMenu :: ~TMainMenu ()
{
    Unload();
}

// loads menu resources
//
void TMainMenu :: Load ()
{
    tstring s;
    if (img_bgr == NULL) {
      s = g_sCurrentDir; s += L"\\gfx\\mainmenu\\000_mmani.454";
      img_bgr = new TImage(s.data(), TRUE/* load compressed */);
    }
    if (img_all == NULL) {
      s = g_sCurrentDir; s += L"\\gfx\\mainmenu\\001_mmall.454";
      img_all = new TImage(s.data(), FALSE/* decompress */);
    }
}

// unloads menu resources to free memory
//
void TMainMenu :: Unload ()
{
    if (img_bgr != NULL) { delete img_bgr; img_bgr = NULL; }
    if (img_all != NULL) { delete img_all; img_all = NULL; }
}

// starts selected level of main menu
//
void TMainMenu :: SetMenuLevel ( int ml )
{
    menu_level  = ml;
    menu_tick   = 0;
    sprnum_back = 0;
    grid_tick   = 0;

    selected_action = 0;

    btn_start_state = 2;
    btn_start_dir   = 0;
    btn_options_state = 2;
    btn_options_dir   = 0;
    btn_about_state = 2;
    btn_about_dir   = 0;
    btn_exit_state  = 2;
    btn_exit_dir    = 0;

    screen_brightness_dir = -1;
}


// draws menu to shadow buffer
//
void TMainMenu :: Draw ()
{
    int   y, i, j;
    DWORD l;
    WORD  c;
    int   a;
    RECT  crect;

    if (surface->graph_out_enabled == FALSE) return;

    switch (menu_level) {
    // level 1 of main menu
    case 1:
        // unpack (draw) background
        y = (screen_dy - img_bgr->dy[sprnum_back]);
        l = (img_bgr->dy[sprnum_back] * img_bgr->dx[sprnum_back])*sizeof(WORD);
        memset(shadow_buffer, 0, y*screen_dx*sizeof(WORD));
        uncompress((LPBYTE)(&(shadow_buffer[y*screen_dx])), &l, (LPBYTE)(img_bgr->data[sprnum_back]), img_bgr->datasize[sprnum_back]);
        // grid
        c = B16(00011101,10001111);
        for (i=0; i<20; i++)
            for (j=0; j<15; j++) {
                a = j+i-grid_tick;
                if (a<5) a = 5;
                if (a<=7) {
                  draw_hline(shadow_buffer, (j<<4), (i<<4), 16, c, a, crect_fullscreen);
                  draw_vline(shadow_buffer, (j<<4), (i<<4)+1, 15, c, a, crect_fullscreen);
                  if (j==14) draw_vline(shadow_buffer, (j<<4)+15, (i<<4)+1, 15, c, a, crect_fullscreen);
                  if (i==19) draw_hline(shadow_buffer, (j<<4), (i<<4)+15, 16, c, a, crect_fullscreen);
                }
            }
        // start button
        crect.left=5; crect.top=5; crect.right=109; crect.bottom=37;
        draw_sprite(shadow_buffer, 5, 5, crect, img_all, btn_start_state, FALSE, FALSE);
        // options button
        crect.left=5; crect.top=37; crect.right=109; crect.bottom=69;
        draw_sprite(shadow_buffer, 5, 5, crect, img_all, btn_options_state, FALSE, FALSE);
        // about button
        crect.left=5; crect.top=69; crect.right=109; crect.bottom=100;
        draw_sprite(shadow_buffer, 5, 5, crect, img_all, btn_about_state, FALSE, FALSE);
        // exit button
        crect.left=5; crect.top=100; crect.right=109; crect.bottom=132;
        draw_sprite(shadow_buffer, 5, 5, crect, img_all, btn_exit_state, FALSE, FALSE);
        break;
    } // switch

    DrawShadowBuffer();
}

// process main menu states
//
void TMainMenu :: TimerTick ()
{
    switch (menu_level) {

    // level 0 (button's action)
    case 0:
      // start/continue game
      if (screen_brightness > 5) {
        if (selected_action == 0) {
          Unload();
          surface->StartGame();
          return;
        }
        // no action selected? how it can be...
        screen_brightness_dir = -1; // fade in
        menu_level = 1; 
      }
      break;

    // level 1 (main)
    case 1:
      // background animation
      sprnum_back++; if (sprnum_back >= img_bgr->count) sprnum_back = 0;
      if (grid_tick<100) grid_tick++;
      // rotating buttons - start
      if ((btn_start_dir == 0) && (btn_start_state > 0)) btn_start_state--;
      if ((btn_start_dir == 1) && (btn_start_state < 2)) btn_start_state++;
      // rotating buttons - options
      if ((btn_options_dir == 0) && (btn_options_state > 0)) btn_options_state--;
      if ((btn_options_dir == 1) && (btn_options_state < 2)) btn_options_state++;
      // rotating buttons - about
      if ((btn_about_dir == 0) && (btn_about_state > 0)) btn_about_state--;
      if ((btn_about_dir == 1) && (btn_about_state < 2)) btn_about_state++;
      // rotating buttons - exit
      if ((btn_exit_dir == 0) && (btn_exit_state > 0)) btn_exit_state--;
      if ((btn_exit_dir == 1) && (btn_exit_state < 2)) btn_exit_state++;
      break;
    }
}

// process mouse clicks on buttons
//
void TMainMenu :: OnMouseDown ( int mx, int my )
{
    switch (menu_level) {

    // level 1 (main)
    case 1:
      if ((mx >= 10) && (mx <= 102) && (my >= 10)  && (my <= 32))  btn_start_dir = 1;
      if ((mx >= 10) && (mx <= 102) && (my >= 42)  && (my <= 63))  btn_options_dir = 1;
      if ((mx >= 10) && (mx <= 102) && (my >= 73)  && (my <= 95))  btn_about_dir = 1;
      if ((mx >= 10) && (mx <= 102) && (my >= 105) && (my <= 126)) btn_exit_dir = 1;
      break;

    } // switch
}

// process mouse clicks on buttons
//
void TMainMenu :: OnMouseUp ( int mx, int my )
{
    switch (menu_level) {

    // level 1 (main)
    case 1:
      // start button
      if ((mx >= 10) && (mx <= 102) && (my >= 10) && (my <= 32) && (btn_start_dir == 1)) {
        menu_level = 0;
        selected_action = 0;
        screen_brightness_dir = 1;
      }
      // options button
      if ((mx >= 10) && (mx <= 102) && (my >= 42) && (my <= 63) && (btn_options_dir == 1)) {
        menu_level = 0;
        selected_action = -1;
        screen_brightness_dir = 1;
      }
      // exit button
      if ((mx >= 10) && (mx <= 102) && (my >= 105) && (my <= 126) && (btn_exit_dir == 1)) {
        SendMessage(g_hMainWindow, WM_DESTROY, 0, 0);
        screen_brightness_dir = 1;
      }
      btn_start_dir = 0;
      btn_options_dir = 0;
      btn_about_dir = 0;
      btn_exit_dir = 0;
      break;

    } // switch
}
